@@ -2,6 +2,7 @@ import curses
 import code
 import sys
 import traceback
+import time
 from contextlib import contextmanager
 import kaa
 from kaa import document
@@ -13,9 +14,9 @@ from kaa.theme import Theme, Style
 from kaa.ui.dialog import dialogmode
 
 pythonconsole_keys = {
-    ('\r'): 'python.exec',
-    (alt, '\r'): 'python.script-history',
-    (alt, '\n'): 'python.script-history',
+    ('\r'): 'pythonconsole.newline',
+    (alt, '\r'): 'pythonconsole.script-history',
+    (alt, '\n'): 'pythonconsole.script-history',
 }
 
 PythonConsoleThemes = {
@@ -97,12 +98,18 @@ class PythonConsoleMode(pythonmode.PythonMode):
         f, t = self.document.marks['current_script']
         wnd.cursor.setpos(f)
 
-    def on_esc_pressed(self, wnd, event):
+    @commandid('pythonconsole.clear')
+    @norec
+    @norerun
+    def clear_line(self, wnd):
         f, t = self.document.marks['current_script']
         self.document.delete(f, t)
         wnd.cursor.setpos(f)
         self.document.undo.clear()
 
+    def on_esc_pressed(self, wnd, event):
+        self.clear_line(wnd)
+
     def put_string(self, wnd, s, overwrite=False):
         pos = wnd.cursor.pos
         f, t = self.document.marks['current_script']
@@ -144,10 +151,19 @@ class PythonConsoleMode(pythonmode.PythonMode):
             def __init__(self, document, style):
                 self.document = document
                 self.style = style
+                self._cache = []
 
             def write(self, s):
-                if not self.document.closed:
-                    self.document.append(s, self.style)
+                if self.document.closed:
+                    return
+                self._cache.append(s)
+                if '\n' in s:
+                    self.flush()
+
+            def flush(self):
+                if self._cache:
+                    self.document.append(''.join(self._cache), self.style)
+                    self._cache = []
 
         sys.stdin = None
         sys.stdout = out(self.document, style_stdout)
@@ -157,30 +173,30 @@ class PythonConsoleMode(pythonmode.PythonMode):
             yield
 
         finally:
+            sys.stdout.flush()
+            sys.stderr.flush()
+
             sys.stdin = stdin
             sys.stdout = stdout
             sys.stderr = stderr
 
-    @commandid('python.exec')
+    @commandid('pythonconsole.exec')
     @norec
     @norerun
-    def on_enter(self, wnd):
-
-        #todo: undo
+    def exec_script(self, wnd):
         f, t = self.document.marks['current_script']
-        tail = wnd.document.gettext(wnd.cursor.pos, t).strip()
-        if tail:
-            self.on_commands(wnd, ['edit.newline'])
-            return
         s = wnd.document.gettext(f, t).lstrip().rstrip(' \t')
         with self._redirect_output(wnd):
             curses.cbreak()
             try:
+                start = time.time()
                 ret = self.interp.runsource(s)
+                end = time.time()
             finally:
                 curses.raw()
 
             if not ret:
+                kaa.app.messagebar.set_message('Execued in {} secs'.format(end-start))
                 wnd.document.undo.clear()
                 if s.strip():
                     hist = kaa.app.config.hist(
@@ -197,6 +213,21 @@ class PythonConsoleMode(pythonmode.PythonMode):
                 self.on_commands(wnd, ['edit.newline'])
                 return
 
+    @commandid('pythonconsole.newline')
+    def on_enter(self, wnd):
+        f, t = self.document.marks['current_script']
+
+        if wnd.cursor.pos < f:
+            wnd.cursor.setpos(f)
+            return
+
+        tail = wnd.document.gettext(wnd.cursor.pos, t).strip()
+        if tail:
+            self.on_commands(wnd, ['edit.newline'])
+            return
+
+        self.exec_script(wnd)
+
     def _put_script(self, wnd, text):
         if text.endswith('\n'):
             text = text[:-1]
@@ -208,7 +239,7 @@ class PythonConsoleMode(pythonmode.PythonMode):
 
     MAX_HISTORY = 100
 
-    @commandid('python.script-history')
+    @commandid('pythonconsole.script-history')
     @norec
     @norerun
     def history(self, wnd):
