@@ -2,7 +2,7 @@ import kaa_testutils
 from kaa import highlight
 
 
-class TestKeyword(kaa_testutils._TestDocBase):
+class TestTokens(kaa_testutils._TestDocBase):
 
     def test_keyword(self):
         kwds = highlight.Keywords(
@@ -295,12 +295,13 @@ class TestHighlight(kaa_testutils._TestDocBase):
         doc = self.get_doc()
         list(hl.highlight(doc, 0))
         doc.styles.setints(14, 19, kwds.tokenid)
+        doc.styles.setints(19, 28, endsection.section_end)
 
         assert 0 == hl.get_resume_pos(doc, 0)
         assert 0 == hl.get_resume_pos(doc, 1)
         assert 14 == hl.get_resume_pos(doc, 16)
         assert 19 == hl.get_resume_pos(doc, 28)
-        assert 19 == hl.get_resume_pos(doc, 29)
+        assert 19 == hl.get_resume_pos(doc, 50)
 
         tokens = list(hl.highlight(doc, 19))
         assert [
